<?php

namespace Drupal\entity_information\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\entity_information\EntityInformationManager;
use Drupal\entity_information\Form\SettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides route responses for entity information.
 */
class EntityInformationController extends ControllerBase {

  /**
   * The entity information manager service.
   *
   * @var \Drupal\entity_information\EntityInformationManager|null
   */
  protected $entityInformationManager;

  /**
   * EntityInformationController constructor.
   *
   * @param \Drupal\entity_information\EntityInformationManager|null $entity_information_manager
   */
  public function __construct(EntityInformationManager $entity_information_manager) {
    $this->entityInformationManager = $entity_information_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.entity_information')
    );
  }

  /**
   * Renders the entity information plugins.
   *
   * @return array
   *   The render array.
   */
  public function render(): array {
    // Get the entity we are building the information plugins for.
    $entity = $this->entityInformationManager->pluggedEntity();

    // Get plugged entity types.
    $plugged_entity_types = $this->entityInformationManager->getPluggedEntityTypes();

    // Get plugin configuration te determine which plugins are enabled.
    $enabled_plugins = $this->config(SettingsForm::SETTINGS)->get('enabled_plugins');

    // Get all available entity information plugins for the current entity.
    $plugins = [];
    if (isset($plugged_entity_types[$entity->getEntityTypeId()][$entity->bundle()])) {
      foreach ($plugged_entity_types[$entity->getEntityTypeId()][$entity->bundle()]['plugins'] as $plugin_name =>$plugin) {
        if (isset($enabled_plugins[$plugin_name]) && $enabled_plugins[$plugin_name] !== 0) {
          $plugins[$plugin_name] = $plugin;
        }
      }
    }

    // Return message if no plugins were found.
    if (count($plugins) === 0) {
      return [
        '#type' => 'markup',
        '#title' => $this->t('Entity information'),
        '#markup' => $this->t('No entity information available. With the right permissions you can enable entity information containers on the @page.', [
          '@page' => Link::createFromRoute('configuration page', 'entity_information.settings')->toString(),
        ]),
      ];
    }

    // Sort plugins by weight.
    uasort($plugins, function($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    // Initiate build array.
    $build = [];

    foreach ($plugins as $plugin_id => $plugin) {
      if ($plugin = $this->entityInformationManager->createInstance($plugin_id, [])) {
        $definitions = $plugin->getPluginDefinition();

        // Create plugin build array.
        $build[$plugin_id]['block'] = [
          '#type' => 'details',
          '#title' => $definitions['label'],
          '#open' => $definitions['open'] ?? TRUE,
          'body' => $plugin->view($entity),
        ];
      }
    }

    return $build;
  }

}
